"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ByteReader = void 0;
var ByteOrder_1 = require("./ByteOrder");
/**
 * This class was heavily influenced by https://github.com/cschwarz/wkx/blob/master/lib/binaryreader.js
 */
var ByteReader = /** @class */ (function () {
    function ByteReader(buffer, byteOrder) {
        if (byteOrder === void 0) { byteOrder = ByteOrder_1.ByteOrder.BIG_ENDIAN; }
        this.buffer = buffer;
        this.position = 0;
        this.setByteOrder(byteOrder);
    }
    ByteReader.prototype._read = function (read, size) {
        return function () {
            var value = read.call(this.buffer, this.position);
            this.position += size;
            return value;
        }.bind(this);
    };
    ByteReader.prototype.readVarInt = function () {
        var nextByte, result = 0, bytesRead = 0;
        do {
            nextByte = this.buffer[this.position + bytesRead];
            result += (nextByte & 0x7F) << (7 * bytesRead);
            bytesRead++;
        } while (nextByte >= 0x80);
        this.position += bytesRead;
        return result;
    };
    ;
    ByteReader.prototype.getByteOrder = function () {
        return this.byteOrder;
    };
    ByteReader.prototype.setByteOrder = function (byteOrder) {
        this.byteOrder = byteOrder;
        if (byteOrder === ByteOrder_1.ByteOrder.BIG_ENDIAN) {
            this.readUInt8 = this._read(Buffer.prototype.readUInt8, 1);
            this.readUInt16 = this._read(Buffer.prototype.readUInt16BE, 2);
            this.readUInt32 = this._read(Buffer.prototype.readUInt32BE, 4);
            this.readInt8 = this._read(Buffer.prototype.readInt8, 1);
            this.readInt16 = this._read(Buffer.prototype.readInt16BE, 2);
            this.readInt32 = this._read(Buffer.prototype.readInt32BE, 4);
            this.readFloat = this._read(Buffer.prototype.readFloatBE, 4);
            this.readDouble = this._read(Buffer.prototype.readDoubleBE, 8);
            this.readByte = this.readUInt8;
            this.readInt = this.readUInt32;
        }
        else {
            this.readUInt8 = this._read(Buffer.prototype.readUInt8, 1);
            this.readUInt16 = this._read(Buffer.prototype.readUInt16LE, 2);
            this.readUInt32 = this._read(Buffer.prototype.readUInt32LE, 4);
            this.readInt8 = this._read(Buffer.prototype.readInt8, 1);
            this.readInt16 = this._read(Buffer.prototype.readInt16LE, 2);
            this.readInt32 = this._read(Buffer.prototype.readInt32LE, 4);
            this.readFloat = this._read(Buffer.prototype.readFloatLE, 4);
            this.readDouble = this._read(Buffer.prototype.readDoubleLE, 8);
            this.readByte = this.readUInt8;
            this.readInt = this.readUInt32;
        }
    };
    return ByteReader;
}());
exports.ByteReader = ByteReader;
//# sourceMappingURL=ByteReader.js.map