"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ByteWriter = void 0;
/**
 * This class was heavily influenced by https://github.com/cschwarz/wkx/blob/master/lib/binarywriter.js
 */
var ByteOrder_1 = require("./ByteOrder");
var ByteWriter = /** @class */ (function () {
    function ByteWriter(size, byteOrder, allowResize) {
        if (byteOrder === void 0) { byteOrder = ByteOrder_1.ByteOrder.BIG_ENDIAN; }
        if (allowResize === void 0) { allowResize = false; }
        this.buffer = Buffer.alloc(size);
        this.position = 0;
        this.allowResize = allowResize;
        this.setByteOrder(byteOrder);
    }
    ByteWriter.prototype._write = function (write, size) {
        return function (value, noAssert) {
            this.ensureSize(size);
            write.call(this.buffer, value, this.position, noAssert);
            this.position += size;
        }.bind(this);
    };
    ByteWriter.prototype.setByteOrder = function (byteOrder) {
        this.byteOrder = byteOrder;
        if (byteOrder === ByteOrder_1.ByteOrder.BIG_ENDIAN) {
            this.writeUInt8 = this._write(Buffer.prototype.writeUInt8, 1);
            this.writeUInt16 = this._write(Buffer.prototype.writeUInt16BE, 2);
            this.writeUInt32 = this._write(Buffer.prototype.writeUInt32BE, 4);
            this.writeInt8 = this._write(Buffer.prototype.writeInt8, 1);
            this.writeInt16 = this._write(Buffer.prototype.writeInt16BE, 2);
            this.writeInt32 = this._write(Buffer.prototype.writeInt32BE, 4);
            this.writeFloat = this._write(Buffer.prototype.writeFloatBE, 4);
            this.writeDouble = this._write(Buffer.prototype.writeDoubleBE, 8);
            this.writeInt = this.writeUInt32;
        }
        else {
            this.writeUInt8 = this._write(Buffer.prototype.writeUInt8, 1);
            this.writeUInt16 = this._write(Buffer.prototype.writeUInt16LE, 2);
            this.writeUInt32 = this._write(Buffer.prototype.writeUInt32LE, 4);
            this.writeInt8 = this._write(Buffer.prototype.writeInt8, 1);
            this.writeInt16 = this._write(Buffer.prototype.writeInt16LE, 2);
            this.writeInt32 = this._write(Buffer.prototype.writeInt32LE, 4);
            this.writeFloat = this._write(Buffer.prototype.writeFloatLE, 4);
            this.writeDouble = this._write(Buffer.prototype.writeDoubleLE, 8);
            this.writeInt = this.writeUInt32;
        }
    };
    ByteWriter.prototype.getByteOrder = function () {
        return this.byteOrder;
    };
    ByteWriter.prototype.writeBuffer = function (buffer) {
        this.ensureSize(buffer.length);
        buffer.copy(this.buffer, this.position, 0, buffer.length);
        this.position += buffer.length;
    };
    ;
    ByteWriter.prototype.writeVarInt = function (value) {
        var length = 1;
        while ((value & 0xFFFFFF80) !== 0) {
            this.writeUInt8((value & 0x7F) | 0x80);
            value >>>= 7;
            length++;
        }
        this.writeUInt8(value & 0x7F);
        return length;
    };
    ;
    ByteWriter.prototype.ensureSize = function (size) {
        if (this.buffer.length < this.position + size) {
            if (this.allowResize) {
                var tempBuffer = Buffer.alloc(this.position + size);
                this.buffer.copy(tempBuffer, 0, 0, this.buffer.length);
                this.buffer = tempBuffer;
            }
            else {
                throw new Error('index out of range');
            }
        }
    };
    ;
    ByteWriter.prototype.getBuffer = function () {
        return this.buffer;
    };
    return ByteWriter;
}());
exports.ByteWriter = ByteWriter;
//# sourceMappingURL=ByteWriter.js.map