import { Geometry, GeometryType } from "@ngageoint/simple-features-js";
/**
 * Geometry Code utilities to convert between geometry attributes and geometry codes
 */
export declare class GeometryCodes {
    /**
     * Get the geometry code from the geometry
     * @param geometry geometry
     * @return geometry code
     */
    static getCode(geometry: Geometry): number;
    /**
     * Get the geometry code from the geometry type
     * @param geometryType geometry type
     * @param hasZ has z
     * @param hasM mas m
     * @return geometry code
     * @since 2.0.3
     */
    static _getCode(geometryType: GeometryType, hasZ: boolean, hasM: boolean): number;
    /**
     * Get the geometry code from the geometry type
     *
     * @param geometryType geometry type
     * @return geometry code
     */
    static getCodeForGeometryType(geometryType: GeometryType): number;
    /**
     * Get the Geometry Type from the code
     * @param code geometry type code
     * @return geometry type
     */
    static getGeometryType(code: number): GeometryType;
    /**
     * Determine if the geometry code has a Z (3D) value
     *
     * @param code
     *            geometry code
     * @return true is has Z
     */
    static hasZ(code: number): boolean;
    /**
     * Determine if the geometry code has a M (linear referencing system) value
     * @param code geometry code
     * @return true is has M
     */
    static hasM(code: number): boolean;
    /**
     * Get the geometry mode from the geometry code. Returns the digit in the
     * thousands place. (z is enabled when 1 or 3, m is enabled when 2 or 3)
     *
     * @param code geometry code
     * @return geometry mode
     */
    static getGeometryMode(code: number): number;
}
