"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GeometryWriter = void 0;
var simple_features_js_1 = require("@ngageoint/simple-features-js");
var ByteWriter_1 = require("./ByteWriter");
var ByteOrder_1 = require("./ByteOrder");
var GeometryCodes_1 = require("./GeometryCodes");
/**
 * Well Known Binary writer
 */
var GeometryWriter = /** @class */ (function () {
    /**
     * Constructor
     * @param writer byte writer
     * @param size
     */
    function GeometryWriter(writer, byteOrder, size) {
        if (byteOrder === void 0) { byteOrder = ByteOrder_1.ByteOrder.BIG_ENDIAN; }
        if (size === void 0) { size = 1024; }
        this.writer = writer || new ByteWriter_1.ByteWriter(size, byteOrder, true);
    }
    /**
     * Write a geometry to a well-known bytes
     *
     * @param geometry geometry
     * @param byteOrder byteOrder
     * @return well-known bytes
     */
    GeometryWriter.writeGeometry = function (geometry, byteOrder) {
        var writer = new GeometryWriter(new ByteWriter_1.ByteWriter(GeometryWriter.getGeometryByteLength(geometry), byteOrder));
        writer.write(geometry);
        return writer.getBuffer();
    };
    /**
     * Get the byte writer
     * @return byte writer
     */
    GeometryWriter.prototype.getByteWriter = function () {
        return this.writer;
    };
    /**
     * Get the written bytes
     *
     * @return written bytes
     */
    GeometryWriter.prototype.getBuffer = function () {
        return this.writer.getBuffer();
    };
    /**
     * Write a geometry to the byte writer
     * @param geometry geometry
     */
    GeometryWriter.prototype.write = function (geometry) {
        this.writer.writeUInt8(this.writer.getByteOrder());
        // Write the geometry type integer
        this.writer.writeInt(GeometryCodes_1.GeometryCodes.getCode(geometry));
        var geometryType = geometry.geometryType;
        switch (geometryType) {
            case simple_features_js_1.GeometryType.GEOMETRY:
                throw new simple_features_js_1.SFException("Unexpected Geometry Type of Geometry which is abstract");
            case simple_features_js_1.GeometryType.POINT:
                this.writePoint(geometry);
                break;
            case simple_features_js_1.GeometryType.LINESTRING:
                this.writeLineString(geometry);
                break;
            case simple_features_js_1.GeometryType.POLYGON:
                this.writePolygon(geometry);
                break;
            case simple_features_js_1.GeometryType.MULTIPOINT:
                this.writeMultiPoint(geometry);
                break;
            case simple_features_js_1.GeometryType.MULTILINESTRING:
                this.writeMultiLineString(geometry);
                break;
            case simple_features_js_1.GeometryType.MULTIPOLYGON:
                this.writeMultiPolygon(geometry);
                break;
            case simple_features_js_1.GeometryType.GEOMETRYCOLLECTION:
            case simple_features_js_1.GeometryType.MULTICURVE:
            case simple_features_js_1.GeometryType.MULTISURFACE:
                this.writeGeometryCollection(geometry);
                break;
            case simple_features_js_1.GeometryType.CIRCULARSTRING:
                this.writeCircularString(geometry);
                break;
            case simple_features_js_1.GeometryType.COMPOUNDCURVE:
                this.writeCompoundCurve(geometry);
                break;
            case simple_features_js_1.GeometryType.CURVEPOLYGON:
                this.writeCurvePolygon(geometry);
                break;
            case simple_features_js_1.GeometryType.CURVE:
                throw new simple_features_js_1.SFException("Unexpected Geometry Type of Curve which is abstract");
            case simple_features_js_1.GeometryType.SURFACE:
                throw new simple_features_js_1.SFException("Unexpected Geometry Type of Surface which is abstract");
            case simple_features_js_1.GeometryType.POLYHEDRALSURFACE:
                this.writePolyhedralSurface(geometry);
                break;
            case simple_features_js_1.GeometryType.TIN:
                this.writeTIN(geometry);
                break;
            case simple_features_js_1.GeometryType.TRIANGLE:
                this.writeTriangle(geometry);
                break;
            default:
                throw new simple_features_js_1.SFException("Geometry Type not supported: " + geometryType);
        }
    };
    GeometryWriter.getGeometryByteLength = function (geometry) {
        var byteLength = GeometryWriter.BYTE_ORDER_BYTE_LENGTH + GeometryWriter.GEOMETRY_TYPE_BYTE_LENGTH;
        var geometryType = geometry.geometryType;
        switch (geometryType) {
            case simple_features_js_1.GeometryType.GEOMETRY:
                throw new simple_features_js_1.SFException("Unexpected Geometry Type of Geometry which is abstract");
            case simple_features_js_1.GeometryType.POINT:
                byteLength += GeometryWriter.getPointByteLength(geometry);
                break;
            case simple_features_js_1.GeometryType.LINESTRING:
                byteLength += GeometryWriter.getLineStringByteLength(geometry);
                break;
            case simple_features_js_1.GeometryType.POLYGON:
                byteLength += GeometryWriter.getPolygonByteLength(geometry);
                break;
            case simple_features_js_1.GeometryType.MULTIPOINT:
                byteLength += GeometryWriter.getMultiPointByteLength(geometry);
                break;
            case simple_features_js_1.GeometryType.MULTILINESTRING:
                byteLength += GeometryWriter.getMultiLineStringByteLength(geometry);
                break;
            case simple_features_js_1.GeometryType.MULTIPOLYGON:
                byteLength += GeometryWriter.getMultiPolygonByteLength(geometry);
                break;
            case simple_features_js_1.GeometryType.GEOMETRYCOLLECTION:
            case simple_features_js_1.GeometryType.MULTICURVE:
            case simple_features_js_1.GeometryType.MULTISURFACE:
                byteLength += GeometryWriter.getGeometryCollectionByteLength(geometry);
                break;
            case simple_features_js_1.GeometryType.CIRCULARSTRING:
                byteLength += GeometryWriter.getCircularStringByteLength(geometry);
                break;
            case simple_features_js_1.GeometryType.COMPOUNDCURVE:
                byteLength += GeometryWriter.getCompoundCurveByteLength(geometry);
                break;
            case simple_features_js_1.GeometryType.CURVEPOLYGON:
                byteLength += GeometryWriter.getCurvePolygonByteLength(geometry);
                break;
            case simple_features_js_1.GeometryType.CURVE:
                throw new simple_features_js_1.SFException("Unexpected Geometry Type of Curve which is abstract");
            case simple_features_js_1.GeometryType.SURFACE:
                throw new simple_features_js_1.SFException("Unexpected Geometry Type of Surface which is abstract");
            case simple_features_js_1.GeometryType.POLYHEDRALSURFACE:
                byteLength += GeometryWriter.getPolyhedralSurfaceByteLength(geometry);
                break;
            case simple_features_js_1.GeometryType.TIN:
                byteLength += GeometryWriter.getTINByteLength(geometry);
                break;
            case simple_features_js_1.GeometryType.TRIANGLE:
                byteLength += GeometryWriter.getTriangleByteLength(geometry);
                break;
            default:
                throw new simple_features_js_1.SFException("Geometry Type not supported: " + geometryType);
        }
        return byteLength;
    };
    /**
     * Write a Point
     * @param point point
     */
    GeometryWriter.prototype.writePoint = function (point) {
        this.writeXY(point);
        this.writeZ(point);
        this.writeM(point);
    };
    /**
     * Write a Point X and Y value
     * @param point point
     */
    GeometryWriter.prototype.writeXY = function (point) {
        this.writer.writeDouble(point.x);
        this.writer.writeDouble(point.y);
    };
    /**
     * Write a Point Z value
     * @param point point
     */
    GeometryWriter.prototype.writeZ = function (point) {
        if (point.hasZ) {
            this.writer.writeDouble(point.z);
        }
    };
    /**
     * Write a Point M value
     * @param point point
     */
    GeometryWriter.prototype.writeM = function (point) {
        if (point.hasM) {
            this.writer.writeDouble(point.m);
        }
    };
    /**
     * Write a Line String
     * @param lineString Line String
     */
    GeometryWriter.prototype.writeLineString = function (lineString) {
        var e_1, _a;
        this.writer.writeInt(lineString.numPoints());
        try {
            for (var _b = __values(lineString.points), _c = _b.next(); !_c.done; _c = _b.next()) {
                var point = _c.value;
                this.writePoint(point);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    /**
     * Write a Polygon
     * @param polygon  Polygon
     */
    GeometryWriter.prototype.writePolygon = function (polygon) {
        var e_2, _a;
        this.writer.writeInt(polygon.numRings());
        try {
            for (var _b = __values(polygon.rings), _c = _b.next(); !_c.done; _c = _b.next()) {
                var ring = _c.value;
                this.writeLineString(ring);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    /**
     * Write a Multi Point
     * @param multiPoint Multi Point
     */
    GeometryWriter.prototype.writeMultiPoint = function (multiPoint) {
        var e_3, _a;
        this.writer.writeInt(multiPoint.numPoints());
        try {
            for (var _b = __values(multiPoint.points), _c = _b.next(); !_c.done; _c = _b.next()) {
                var point = _c.value;
                this.write(point);
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
    };
    /**
     * Write a Multi Line String
     * @param multiLineString Multi Line String
     */
    GeometryWriter.prototype.writeMultiLineString = function (multiLineString) {
        var e_4, _a;
        this.writer.writeInt(multiLineString.numLineStrings());
        try {
            for (var _b = __values(multiLineString.lineStrings), _c = _b.next(); !_c.done; _c = _b.next()) {
                var lineString = _c.value;
                this.write(lineString);
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_4) throw e_4.error; }
        }
    };
    /**
     * Write a Multi Polygon
     * @param multiPolygon Multi Polygon
     */
    GeometryWriter.prototype.writeMultiPolygon = function (multiPolygon) {
        var e_5, _a;
        this.writer.writeInt(multiPolygon.numPolygons());
        try {
            for (var _b = __values(multiPolygon.polygons), _c = _b.next(); !_c.done; _c = _b.next()) {
                var polygon = _c.value;
                this.write(polygon);
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_5) throw e_5.error; }
        }
    };
    /**
     * Write a Geometry Collection
     * @param geometryCollection Geometry Collection
     */
    GeometryWriter.prototype.writeGeometryCollection = function (geometryCollection) {
        var e_6, _a;
        this.writer.writeInt(geometryCollection.numGeometries());
        try {
            for (var _b = __values(geometryCollection.geometries), _c = _b.next(); !_c.done; _c = _b.next()) {
                var geometry = _c.value;
                this.write(geometry);
            }
        }
        catch (e_6_1) { e_6 = { error: e_6_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_6) throw e_6.error; }
        }
    };
    /**
     * Write a Circular String
     * @param circularString Circular String
     */
    GeometryWriter.prototype.writeCircularString = function (circularString) {
        var e_7, _a;
        this.writer.writeInt(circularString.numPoints());
        try {
            for (var _b = __values(circularString.points), _c = _b.next(); !_c.done; _c = _b.next()) {
                var point = _c.value;
                this.writePoint(point);
            }
        }
        catch (e_7_1) { e_7 = { error: e_7_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_7) throw e_7.error; }
        }
    };
    /**
     * Write a Compound Curve
     * @param compoundCurve Compound Curve
     */
    GeometryWriter.prototype.writeCompoundCurve = function (compoundCurve) {
        var e_8, _a;
        this.writer.writeInt(compoundCurve.numLineStrings());
        try {
            for (var _b = __values(compoundCurve.lineStrings), _c = _b.next(); !_c.done; _c = _b.next()) {
                var lineString = _c.value;
                this.write(lineString);
            }
        }
        catch (e_8_1) { e_8 = { error: e_8_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_8) throw e_8.error; }
        }
    };
    /**
     * Write a Curve Polygon
     * @param curvePolygon Curve Polygon
     */
    GeometryWriter.prototype.writeCurvePolygon = function (curvePolygon) {
        var e_9, _a;
        this.writer.writeInt(curvePolygon.numRings());
        try {
            for (var _b = __values(curvePolygon.rings), _c = _b.next(); !_c.done; _c = _b.next()) {
                var ring = _c.value;
                this.write(ring);
            }
        }
        catch (e_9_1) { e_9 = { error: e_9_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_9) throw e_9.error; }
        }
    };
    /**
     * Write a Polyhedral Surface
     * @param polyhedralSurface Polyhedral Surface
     */
    GeometryWriter.prototype.writePolyhedralSurface = function (polyhedralSurface) {
        var e_10, _a;
        this.writer.writeInt(polyhedralSurface.numPolygons());
        try {
            for (var _b = __values(polyhedralSurface.polygons), _c = _b.next(); !_c.done; _c = _b.next()) {
                var polygon = _c.value;
                this.write(polygon);
            }
        }
        catch (e_10_1) { e_10 = { error: e_10_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_10) throw e_10.error; }
        }
    };
    /**
     * Write a TIN
     * @param tin TIN
     */
    GeometryWriter.prototype.writeTIN = function (tin) {
        var e_11, _a;
        this.writer.writeInt(tin.numPolygons());
        try {
            for (var _b = __values(tin.polygons), _c = _b.next(); !_c.done; _c = _b.next()) {
                var polygon = _c.value;
                this.write(polygon);
            }
        }
        catch (e_11_1) { e_11 = { error: e_11_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_11) throw e_11.error; }
        }
    };
    /**
     * Write a Triangle
     * @param triangle Triangle
     */
    GeometryWriter.prototype.writeTriangle = function (triangle) {
        var e_12, _a;
        this.writer.writeInt(triangle.numRings());
        try {
            for (var _b = __values(triangle.rings), _c = _b.next(); !_c.done; _c = _b.next()) {
                var ring = _c.value;
                this.writeLineString(ring);
            }
        }
        catch (e_12_1) { e_12 = { error: e_12_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_12) throw e_12.error; }
        }
    };
    /**
     * Write a geometry to the byte writer
     * @param writer  byte writer
     * @param geometry geometry
     */
    GeometryWriter.writeGeometryWithByteWriter = function (writer, geometry) {
        var geometryWriter = new GeometryWriter(writer);
        geometryWriter.write(geometry);
    };
    /**
     * Write a Point
     *
     * @param writer
     *            byte writer
     * @param point
     *            point
     * @throws IOException
     *             upon failure to write
     */
    GeometryWriter.writePoint = function (writer, point) {
        var geometryWriter = new GeometryWriter(writer);
        geometryWriter.writePoint(point);
    };
    /**
     * Write a Line String
     *
     * @param writer
     *            byte writer
     * @param lineString
     *            Line String
     */
    GeometryWriter.writeLineString = function (writer, lineString) {
        var geometryWriter = new GeometryWriter(writer);
        geometryWriter.writeLineString(lineString);
    };
    /**
     * Write a Polygon
     *
     * @param writer
     *            byte writer
     * @param polygon
     *            Polygon
     * @throws IOException
     *             upon failure to write
     */
    GeometryWriter.writePolygon = function (writer, polygon) {
        var geometryWriter = new GeometryWriter(writer);
        geometryWriter.writePolygon(polygon);
    };
    /**
     * Write a Multi Point
     *
     * @param writer
     *            byte writer
     * @param multiPoint
     *            Multi Point
     * @throws IOException
     *             upon failure to write
     */
    GeometryWriter.writeMultiPoint = function (writer, multiPoint) {
        var geometryWriter = new GeometryWriter(writer);
        geometryWriter.writeMultiPoint(multiPoint);
    };
    /**
     * Write a Multi Line String
     *
     * @param writer
     *            byte writer
     * @param multiLineString
     *            Multi Line String
     * @throws IOException
     *             upon failure to write
     */
    GeometryWriter.writeMultiLineString = function (writer, multiLineString) {
        var geometryWriter = new GeometryWriter(writer);
        geometryWriter.writeMultiLineString(multiLineString);
    };
    /**
     * Write a Multi Polygon
     *
     * @param writer
     *            byte writer
     * @param multiPolygon
     *            Multi Polygon
     * @throws IOException
     *             upon failure to write
     */
    GeometryWriter.writeMultiPolygon = function (writer, multiPolygon) {
        var geometryWriter = new GeometryWriter(writer);
        geometryWriter.writeMultiPolygon(multiPolygon);
    };
    /**
     * Write a Geometry Collection
     *
     * @param writer
     *            byte writer
     * @param geometryCollection
     *            Geometry Collection
     * @throws IOException
     *             upon failure to write
     */
    GeometryWriter.writeGeometryCollection = function (writer, geometryCollection) {
        var geometryWriter = new GeometryWriter(writer);
        geometryWriter.writeGeometryCollection(geometryCollection);
    };
    /**
     * Write a Circular String
     *
     * @param writer
     *            byte writer
     * @param circularString
     *            Circular String
     * @throws IOException
     *             upon failure to write
     */
    GeometryWriter.writeCircularString = function (writer, circularString) {
        var geometryWriter = new GeometryWriter(writer);
        geometryWriter.writeCircularString(circularString);
    };
    /**
     * Write a Compound Curve
     *
     * @param writer
     *            byte writer
     * @param compoundCurve
     *            Compound Curve
     * @throws IOException
     *             upon failure to write
     */
    GeometryWriter.writeCompoundCurve = function (writer, compoundCurve) {
        var geometryWriter = new GeometryWriter(writer);
        geometryWriter.writeCompoundCurve(compoundCurve);
    };
    /**
     * Write a Curve Polygon
     *
     * @param writer
     *            byte writer
     * @param curvePolygon
     *            Curve Polygon
     * @throws IOException
     *             upon failure to write
     */
    GeometryWriter.writeCurvePolygon = function (writer, curvePolygon) {
        var geometryWriter = new GeometryWriter(writer);
        geometryWriter.writeCurvePolygon(curvePolygon);
    };
    /**
     * Write a Polyhedral Surface
     *
     * @param writer
     *            byte writer
     * @param polyhedralSurface
     *            Polyhedral Surface
     * @throws IOException
     *             upon failure to write
     */
    GeometryWriter.writePolyhedralSurface = function (writer, polyhedralSurface) {
        var geometryWriter = new GeometryWriter(writer);
        geometryWriter.writePolyhedralSurface(polyhedralSurface);
    };
    /**
     * Write a TIN
     *
     * @param writer
     *            byte writer
     * @param tin
     *            TIN
     * @throws IOException
     *             upon failure to write
     */
    GeometryWriter.writeTIN = function (writer, tin) {
        var geometryWriter = new GeometryWriter(writer);
        geometryWriter.writeTIN(tin);
    };
    /**
     * Write a Triangle
     *
     * @param writer
     *            byte writer
     * @param triangle
     *            Triangle
     * @throws IOException
     *             upon failure to write
     */
    GeometryWriter.writeTriangle = function (writer, triangle) {
        var geometryWriter = new GeometryWriter(writer);
        geometryWriter.writeTriangle(triangle);
    };
    GeometryWriter.getPointByteLength = function (point) {
        var byteLength = GeometryWriter.DOUBLE_BYTE_LENGTH // x
            + GeometryWriter.DOUBLE_BYTE_LENGTH; // y
        if (point.hasZ) {
            byteLength += GeometryWriter.DOUBLE_BYTE_LENGTH;
        }
        if (point.hasM) {
            byteLength += GeometryWriter.DOUBLE_BYTE_LENGTH;
        }
        return byteLength;
    };
    /**
     * Write a Line String
     * @param lineString Line String
     */
    GeometryWriter.getLineStringByteLength = function (lineString) {
        var e_13, _a;
        var byteLength = GeometryWriter.INT_BYTE_LENGTH;
        try {
            for (var _b = __values(lineString.points), _c = _b.next(); !_c.done; _c = _b.next()) {
                var point = _c.value;
                byteLength += GeometryWriter.getPointByteLength(point);
            }
        }
        catch (e_13_1) { e_13 = { error: e_13_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_13) throw e_13.error; }
        }
        return byteLength;
    };
    /**
     * Write a Polygon
     * @param polygon Polygon
     */
    GeometryWriter.getPolygonByteLength = function (polygon) {
        var e_14, _a;
        var byteLength = GeometryWriter.INT_BYTE_LENGTH;
        try {
            for (var _b = __values(polygon.rings), _c = _b.next(); !_c.done; _c = _b.next()) {
                var ring = _c.value;
                byteLength += GeometryWriter.getLineStringByteLength(ring);
            }
        }
        catch (e_14_1) { e_14 = { error: e_14_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_14) throw e_14.error; }
        }
        return byteLength;
    };
    /**
     * Write a Multi Point
     * @param multiPoint Multi Point
     */
    GeometryWriter.getMultiPointByteLength = function (multiPoint) {
        var e_15, _a;
        var byteLength = GeometryWriter.INT_BYTE_LENGTH;
        try {
            for (var _b = __values(multiPoint.points), _c = _b.next(); !_c.done; _c = _b.next()) {
                var point = _c.value;
                byteLength += GeometryWriter.getGeometryByteLength(point);
            }
        }
        catch (e_15_1) { e_15 = { error: e_15_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_15) throw e_15.error; }
        }
        return byteLength;
    };
    /**
     * Write a Multi Line String
     * @param multiLineString Multi Line String
     */
    GeometryWriter.getMultiLineStringByteLength = function (multiLineString) {
        var e_16, _a;
        var byteLength = GeometryWriter.INT_BYTE_LENGTH;
        try {
            for (var _b = __values(multiLineString.lineStrings), _c = _b.next(); !_c.done; _c = _b.next()) {
                var lineString = _c.value;
                byteLength += GeometryWriter.getGeometryByteLength(lineString);
            }
        }
        catch (e_16_1) { e_16 = { error: e_16_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_16) throw e_16.error; }
        }
        return byteLength;
    };
    /**
     * Write a Multi Polygon
     * @param multiPolygon Multi Polygon
     */
    GeometryWriter.getMultiPolygonByteLength = function (multiPolygon) {
        var e_17, _a;
        var byteLength = GeometryWriter.INT_BYTE_LENGTH;
        try {
            for (var _b = __values(multiPolygon.polygons), _c = _b.next(); !_c.done; _c = _b.next()) {
                var polygon = _c.value;
                byteLength += GeometryWriter.getGeometryByteLength(polygon);
            }
        }
        catch (e_17_1) { e_17 = { error: e_17_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_17) throw e_17.error; }
        }
        return byteLength;
    };
    /**
     * Write a Geometry Collection
     * @param geometryCollection Geometry Collection
     */
    GeometryWriter.getGeometryCollectionByteLength = function (geometryCollection) {
        var e_18, _a;
        var byteLength = GeometryWriter.INT_BYTE_LENGTH;
        try {
            for (var _b = __values(geometryCollection.geometries), _c = _b.next(); !_c.done; _c = _b.next()) {
                var geometry = _c.value;
                byteLength += GeometryWriter.getGeometryByteLength(geometry);
            }
        }
        catch (e_18_1) { e_18 = { error: e_18_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_18) throw e_18.error; }
        }
        return byteLength;
    };
    /**
     * Write a Circular String
     * @param circularString Circular String
     */
    GeometryWriter.getCircularStringByteLength = function (circularString) {
        var e_19, _a;
        var byteLength = GeometryWriter.INT_BYTE_LENGTH;
        try {
            for (var _b = __values(circularString.points), _c = _b.next(); !_c.done; _c = _b.next()) {
                var point = _c.value;
                byteLength += GeometryWriter.getPointByteLength(point);
            }
        }
        catch (e_19_1) { e_19 = { error: e_19_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_19) throw e_19.error; }
        }
        return byteLength;
    };
    /**
     * Write a Compound Curve
     * @param compoundCurve Compound Curve
     */
    GeometryWriter.getCompoundCurveByteLength = function (compoundCurve) {
        var e_20, _a;
        var byteLength = GeometryWriter.INT_BYTE_LENGTH;
        try {
            for (var _b = __values(compoundCurve.lineStrings), _c = _b.next(); !_c.done; _c = _b.next()) {
                var lineString = _c.value;
                byteLength += GeometryWriter.getGeometryByteLength(lineString);
            }
        }
        catch (e_20_1) { e_20 = { error: e_20_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_20) throw e_20.error; }
        }
        return byteLength;
    };
    /**
     * Write a Curve Polygon
     * @param curvePolygon Curve Polygon
     */
    GeometryWriter.getCurvePolygonByteLength = function (curvePolygon) {
        var e_21, _a;
        var byteLength = GeometryWriter.INT_BYTE_LENGTH;
        try {
            for (var _b = __values(curvePolygon.rings), _c = _b.next(); !_c.done; _c = _b.next()) {
                var ring = _c.value;
                byteLength += GeometryWriter.getGeometryByteLength(ring);
            }
        }
        catch (e_21_1) { e_21 = { error: e_21_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_21) throw e_21.error; }
        }
        return byteLength;
    };
    /**
     * Write a Polyhedral Surface
     * @param polyhedralSurface Polyhedral Surface
     */
    GeometryWriter.getPolyhedralSurfaceByteLength = function (polyhedralSurface) {
        var e_22, _a;
        var byteLength = GeometryWriter.INT_BYTE_LENGTH;
        try {
            for (var _b = __values(polyhedralSurface.polygons), _c = _b.next(); !_c.done; _c = _b.next()) {
                var polygon = _c.value;
                byteLength += GeometryWriter.getGeometryByteLength(polygon);
            }
        }
        catch (e_22_1) { e_22 = { error: e_22_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_22) throw e_22.error; }
        }
        return byteLength;
    };
    /**
     * Write a TIN
     * @param tin TIN
     */
    GeometryWriter.getTINByteLength = function (tin) {
        var e_23, _a;
        var byteLength = GeometryWriter.INT_BYTE_LENGTH;
        try {
            for (var _b = __values(tin.polygons), _c = _b.next(); !_c.done; _c = _b.next()) {
                var polygon = _c.value;
                byteLength += GeometryWriter.getGeometryByteLength(polygon);
            }
        }
        catch (e_23_1) { e_23 = { error: e_23_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_23) throw e_23.error; }
        }
        return byteLength;
    };
    /**
     * Write a Triangle
     * @param triangle Triangle
     */
    GeometryWriter.getTriangleByteLength = function (triangle) {
        var e_24, _a;
        var byteLength = GeometryWriter.INT_BYTE_LENGTH;
        try {
            for (var _b = __values(triangle.rings), _c = _b.next(); !_c.done; _c = _b.next()) {
                var ring = _c.value;
                byteLength += GeometryWriter.getLineStringByteLength(ring);
            }
        }
        catch (e_24_1) { e_24 = { error: e_24_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_24) throw e_24.error; }
        }
        return byteLength;
    };
    GeometryWriter.BYTE_ORDER_BYTE_LENGTH = 1;
    GeometryWriter.GEOMETRY_TYPE_BYTE_LENGTH = 4;
    GeometryWriter.INT_BYTE_LENGTH = 4;
    GeometryWriter.DOUBLE_BYTE_LENGTH = 8;
    return GeometryWriter;
}());
exports.GeometryWriter = GeometryWriter;
//# sourceMappingURL=GeometryWriter.js.map